@@ -1,9 +1,10 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+    Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -24,7 +24,7 @@
 #include "rectangle.h"
 #include "ucs.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "character.h"
 #include "profile.h"
 #include "feats.h"
@@ -34,32 +34,32 @@
 //
 void Character::recognize1( const Charset & charset, const Rectangle & charbox ) throw()
   {
-  if( blocks() == 1 ) recognize11( charset, charbox );
-  else if( blocks() == 2 ) recognize12( charset, charbox );
-  else if( blocks() == 3 ) recognize13( charset, charbox );
+  if( blobs() == 1 ) recognize11( charset, charbox );
+  else if( blobs() == 2 ) recognize12( charset, charbox );
+  else if( blobs() == 3 ) recognize13( charset, charbox );
   }
 
 
-// Recognizes 1 block characters.
+// Recognizes 1 blob characters.
 // 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghklmnopqrstuvwxyz
 // #$&'()*+,-./<>@[\]^_`{|}~
 //
 void Character::recognize11( const Charset & charset, const Rectangle & charbox ) throw()
   {
-  const Block & b = block( 0 );
+  const Blob & b = blob( 0 );
   if( b.holes() == 0 ) recognize110( charset, charbox );
   else if( b.holes() == 1 ) recognize111( charset, charbox );
   else if( b.holes() == 2 ) recognize112( charbox );
   }
 
 
-// Recognizes 1 block characters without holes.
+// Recognizes 1 blob characters without holes.
 // 12357CEFGHIJKLMNSTUVWXYZcfhklmnrstuvwxyz
 // '()*+,-./<>@[\]^_`{|}~
 //
 void Character::recognize110( const Charset & charset, const Rectangle & charbox ) throw()
   {
-  const Block & b = block( 0 );
+  const Blob & b = blob( 0 );
   Features f( b );
   int code = f.test_easy( charbox );
   if( code )
@@ -69,15 +69,14 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
     add_guess( code, 0 ); return;
     }
   if( b.height() < 5 || ( b.height() < 8 && b.width() < 6 ) ||
-      ( b.height() > 10 * b.width() ) ||
-      ( b.height() * 5 < b.width() ) ) return;
+      b.height() > 10 * b.width() || 5 * b.height() < b.width() ) return;
 
   code = f.test_CEFIJLlT( charset ); if( code ) { add_guess( code, 0 ); return; }
-  code = f.test_frst();              if( code ) { add_guess( code, 0 ); return; }
+  code = f.test_frst( charbox );     if( code ) { add_guess( code, 0 ); return; }
   code = f.test_G();                 if( code ) { add_guess( code, 0 ); return; }
   code = f.test_c();                 if( code ) { add_guess( code, 0 ); return; }
   if( charset.enabled( Charset::iso_8859_9 ) )
-    { code = f.test_s_cedilla(); if( code ) { add_guess( code, 0 ); return; } }
+    { code = f.test_s_cedilla();     if( code ) { add_guess( code, 0 ); return; } }
   code = f.test_235Esz( charset );   if( code ) { add_guess( code, 0 ); return; }
   code = f.test_HKMNUuvwYy( charbox );
   if( code == 'u' && f.lp.istpit() )	// Looks for merged 'tr'
@@ -88,27 +87,46 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
       col = b.seek_left( b.vcenter(), col - 1, false );
       while( --col > b.hpos( 40 ) &&
              ( b.seek_top( b.vcenter(), col ) > b.top() ||
-               f.hp[col-b.left()] > b.height() / 10 ) );
+               f.hp[col-b.left()] > b.height() / 10 ) ) ;
       if( col > b.hpos( 40 ) && col < b.right() )
-        { only_guess( 0, b.left() ); add_guess( 't', col );
-        add_guess( 'r', b.right() ); return; }
+        {
+        only_guess( 0, b.left() ); add_guess( 't', col );
+        add_guess( 'r', b.right() ); return;
+        }
+      }
+    }
+  if( code == 'N' && b.width() > b.height() && b.top() >= charbox.top() &&
+      4 * f.tp[f.tp.pos(50)] < b.height() )
+    {					// Looks for merged 'rv'
+    const int col = f.hp.iminimum();
+    if( col >= f.hp.pos( 40 ) && col < f.hp.pos( 50 ) )
+      {
+      only_guess( 0, b.left() ); add_guess ( 'r', b.left() + col );
+      add_guess( 'v', b.right() ); return;
       }
     }
   if( code ) { add_guess( code, 0 ); return; }
-  if( f.bp.minima( b.height() / 8 ) == 2 )
+  const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
+  if( f.bp.minima() <= 2 &&
+      ( f.bp.minima( b.height() / 8 + noise ) == 2 ||
+        ( b.height() >= 16 && f.bp.minima( b.height() / 8 ) == 2 ) ) )
     {
-    code = f.test_hknwx();
-    if( code == 'n' )	// Looks for '"' or merged 'rr' or 'rt' or 'fl'
+    code = f.test_hknwx( charbox );
+    if( code == 'n' )	// Looks for '"' or merged 'rr' or 'TT' or 'rt' or 'fl'
       {
       if( b.bottom() <= charbox.vcenter() ) { add_guess( '"', 0 ); return; }
       if( b.width() > b.height() && 10 * f.lp[f.lp.pos(10)] < b.width() &&
           !f.rp.increasing( f.rp.pos( 75 ) ) )
         {
-        const int col = b.seek_left( b.vcenter(), b.right() );
-        if( col < b.hpos( 90 ) && !b.escape_top( b.vcenter(), col ) )
+        const int rgap = f.rp[f.rp.pos(50)];
+        if( 10 * rgap > b.width() && !b.escape_top( b.vcenter(), b.right() ) )
           {
           only_guess( 0, b.left() );
-          add_guess( 'r', b.hcenter() ); add_guess( 'r', b.right() );
+          const int lgap = f.lp[f.lp.pos(50)];
+          if( 10 * lgap > b.width() && !b.escape_top( b.vcenter(), b.left() ) &&
+              Ocrad::similar( lgap, rgap, 10 ) && b.top() < charbox.top() )
+            { add_guess( 'T', b.hcenter() ); add_guess( 'T', b.right() ); }
+          else { add_guess( 'r', b.hcenter() ); add_guess( 'r', b.right() ); }
           return;
           }
         }
@@ -125,7 +143,7 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
       if( f.rp.minima() == 1 && !f.rp.increasing( f.rp.pos( 75 ) ) )
         {
         int dmax = 0; bool bar = false;
-        for( int row = b.vcenter(); row > b.vpos( 25 ); --row )
+        for( int row = b.vpos( 60 ); row > b.vpos( 25 ); --row )
           {
           int d = b.hcenter() - b.seek_left( row, b.hcenter() );
           if( d > dmax ) dmax = d;
@@ -162,7 +180,7 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
           }
         return;
         }
-      if( f.rp.isctip( 33 ) )
+      if( f.rp.isctip( 30 ) )
         {
         only_guess( 0, b.left() ); add_guess( 'f', b.hcenter() );
         add_guess( 'i', b.right() ); return;
@@ -180,9 +198,12 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
     }
   if( f.bp.minima() == 3 )
     {
-    if( f.tp.minima() == 3 && f.lp.minima() == 2 && f.rp.minima() == 2 )
+    if( f.bp.minima( b.height() / 2 ) == 1 && f.tp.minima() == 3 &&
+        f.lp.minima() == 2 && f.rp.minima() == 2 )
       { add_guess( '*', 0 ); return; }
     if( b.id( b.vcenter(), b.hcenter() ) == 0 &&
+        b.id( b.vcenter() - 1, b.hcenter() ) == 0 &&
+        b.id( b.vcenter() + 1, b.hcenter() ) == 0 &&
         b.seek_left( b.vcenter(), b.hcenter() ) <= b.hpos( 25 ) )
       {					// Found merged 'rn'
       int row = b.vpos( 95 );
@@ -219,17 +240,17 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
   code = f.test_line( charbox );
   if( code ) { add_guess( code, 0 ); return; }
 
-  code = f.test_misc();
+  code = f.test_misc( charbox );
   if( code ) { add_guess( code, 0 ); return; }
   }
 
 
-// Recognizes 1 block characters with 1 hole.
+// Recognizes 1 blob characters with 1 hole.
 // 0469ADOPQRabdegopq#
 //
 void Character::recognize111( const Charset & charset, const Rectangle & charbox ) throw()
   {
-  const Block & b = block( 0 );
+  const Blob & b = blob( 0 );
   const Bitmap & h = b.hole( 0 );
   if( !h.is_hcentred_in( b ) ) return;
   Features f( b );
@@ -265,22 +286,22 @@ void Character::recognize111( const Charset & charset, const Rectangle & charbox
         {
         int row = h.top() - ( b.bottom() - h.bottom() ) - 1;
         if( row <= b.top() || row + 1 >= h.top() ) return;
-        Block & b1 = const_cast< Block & >( b );
-        Block b2( b );
+        Blob & b1 = const_cast< Blob & >( b );
+        Blob b2( b );
         b1.bottom( row ); b2.top( row + 1 );
-        bpv.push_back( new Block( b2 ) );
+        bpv.push_back( new Blob( b2 ) );
         }
       }
     }
   }
 
 
-// Recognizes 1 block characters with 2 holes.
+// Recognizes 1 blob characters with 2 holes.
 // 8BQg$&
 //
 void Character::recognize112( const Rectangle & charbox ) throw()
   {
-  const Block & b = block( 0 );
+  const Blob & b = blob( 0 );
   const Bitmap & h1 = b.hole( 0 );		// upper hole
   const Bitmap & h2 = b.hole( 1 );		// lower hole
   Profile lp( b, Profile::left );
@@ -310,6 +331,13 @@ void Character::recognize112( const Rectangle & charbox ) throw()
   const int a1 = h1.area();
   const int a2 = h2.area();
 
+  {
+  const int w = b.right() - std::min( b.hcenter(),
+                            std::min( h1.hcenter(), h2.hcenter() ) );
+  for( int i = h1.bottom() - b.top() + 1; i < h2.top() - b.top(); ++i )
+    if( rp[i] > w ) { add_guess( 'g', 2 ); return; }
+  }
+
   if( Ocrad::similar( a1, a2, 50 ) )		// I don't like this
     {
     if( h1.bottom() > b.vcenter() && h2.top() < b.vcenter() &&
@@ -324,9 +352,6 @@ void Character::recognize112( const Rectangle & charbox ) throw()
     
       if( lp.isflats() && !lp.ispit() ) { add_guess( 'B', 0 ); return; }
 
-      for( int i = h1.bottom() - b.top() + 1; i < h2.top() - b.top(); ++i )
-        if( 2 * rp[i] > b.width() ) { add_guess( 'g', 2 ); return; }
-
       int col1 = h1.seek_left( h1.bottom(), h1.right() + 1 ) - 1;
       int col2 = h2.seek_right( h2.top(), h2.left() - 1 ) + 1;
       if( col1 <= col2 )
@@ -347,12 +372,14 @@ void Character::recognize112( const Rectangle & charbox ) throw()
       if( charbox.bottom() > h2.vcenter() &&
           ( bp.isconvex() || ( bp.ispit() && tp.ispit() ) ) )
         {
-        if( b.top() >= charbox.top() && ( lp.ispit() || lp.isconvex() ) &&
-            ( !rp.ispit() || h2.right() > h1.right() ) )
-          add_guess( 'e', 1 );
-        else if( b.top() >= charbox.top() &&
-                 b.right() - rp[rp.pos(50)] > h1.right() && !rp.isctip() )
-          add_guess( 'a', 1 );
+        if( b.top() >= charbox.top() && b.height() <= charbox.height() )
+          {
+          if( ( lp.ispit() || lp.isconvex() ) &&
+              ( !rp.ispit() || h2.right() > h1.right() ) )
+            add_guess( 'e', 1 );
+          else if( b.right() - rp[rp.pos(50)] > h1.right() && !rp.isctip() )
+            add_guess( 'a', 1 );
+          }
         if( h1.bottom() > b.vcenter() && h1.top() > b.vpos( 30 ) )
           add_guess( UCS::SEACUTE, 0 );
         else add_guess( '8', 0 );
@@ -368,7 +395,7 @@ void Character::recognize112( const Rectangle & charbox ) throw()
         }
 
       if( charbox.vcenter() > h1.top() &&
-          ( charbox.vcenter() <= h1.bottom() || charbox.bottom() < h2.vcenter() ) )
+          ( charbox.vcenter() < h1.bottom() || charbox.bottom() < h2.vcenter() ) )
         add_guess( 'g', 2 );
       add_guess( 'B', 1 ); add_guess( 'a', 0 );
       return;
