@@ -1,9 +1,10 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+    Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,13 +13,14 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
 #include <cstdio>
 #include <cstdlib>
+#include <cstring>
+
 #include "common.h"
 
 
@@ -32,6 +34,22 @@ const Charset::Value charset_value[charsets] =
 const char * const charset_name[charsets] =
   { "ascii", "iso-8859-9", "iso-8859-15" };
 
+struct F_entry
+  {
+  const char * name;
+  Filter::Type type;
+  };
+
+const F_entry F_table[] =
+  {
+  { "none",         Filter::none },
+  { "letters",      Filter::letters },
+  { "letters_only", Filter::letters_only },
+  { "numbers",      Filter::numbers },
+  { "numbers_only", Filter::numbers_only },
+  { 0, Filter::none }
+  };
+
 struct T_entry
   {
   const char * name;
@@ -67,33 +85,59 @@ bool Charset::enable( const char * name ) throw()
   {
   for( int i = 0; i < charsets; ++i )
     if( std::strcmp( name, charset_name[i] ) == 0 )
-      { _charset |= charset_value[i]; return true; }
+      { charset_ |= charset_value[i]; return true; }
   return false;
   }
 
 
 bool Charset::enabled( Value cset ) const throw()
   {
-  if( !_charset ) return cset == iso_8859_15;		// default charset
-  return _charset & cset;
+  if( !charset_ ) return cset == iso_8859_15;		// default charset
+  return charset_ & cset;
   }
 
 
 bool Charset::only( Value cset ) const throw()
   {
-  if( !_charset ) return cset == iso_8859_15;		// default charset
-  return _charset == cset;
+  if( !charset_ ) return cset == iso_8859_15;		// default charset
+  return charset_ == cset;
   }
 
 
 void Charset::show_error( const char * program_name, const char * arg ) const throw()
   {
-  if( arg && std::strcmp( arg, "help" ) )
-    std::fprintf( stderr,"%s: bad charset '%s'\n", program_name, arg );
-  std::fputs( "Valid charset names are:", stderr );
-  for( int i = 0; i < charsets; ++i )
-    std::fprintf( stderr, "  %s", charset_name[i] );
-  std::fputs( "\n", stderr );
+  if( verbosity >= 0 )
+    {
+    if( arg && std::strcmp( arg, "help" ) )
+      std::fprintf( stderr,"%s: bad charset `%s'\n", program_name, arg );
+    std::fputs( "Valid charset names are:", stderr );
+    for( int i = 0; i < charsets; ++i )
+      std::fprintf( stderr, "  %s", charset_name[i] );
+    std::fputs( "\n", stderr );
+    }
+  }
+
+
+bool Filter::set( const char * name ) throw()
+  {
+  for( int i = 0; F_table[i].name != 0; ++i )
+    if( std::strcmp( name, F_table[i].name ) == 0 )
+      { type_ = F_table[i].type; return true; }
+  return false;
+  }
+
+
+void Filter::show_error( const char * program_name, const char * arg ) const throw()
+  {
+  if( verbosity >= 0 )
+    {
+    if( arg && std::strcmp( arg, "help" ) )
+      std::fprintf( stderr,"%s: bad filter `%s'\n", program_name, arg );
+    std::fputs( "Valid filter names are:", stderr );
+    for( int i = 0; F_table[i].name != 0; ++i )
+      std::fprintf( stderr, "  %s", F_table[i].name );
+    std::fputs( "\n", stderr );
+    }
   }
 
 
@@ -101,19 +145,22 @@ bool Transformation::set( const char * name ) throw()
   {
   for( int i = 0; T_table[i].name != 0; ++i )
     if( std::strcmp( name, T_table[i].name ) == 0 )
-      { _type = T_table[i].type; return true; }
+      { type_ = T_table[i].type; return true; }
   return false;
   }
 
 
 void Transformation::show_error( const char * program_name, const char * arg ) const throw()
   {
-  if( arg && std::strcmp( arg, "help" ) )
-    std::fprintf( stderr,"%s: bad bitmap trasformation '%s'\n", program_name, arg );
-  std::fputs( "Valid transformation names are:", stderr );
-  for( int i = 0; T_table[i].name != 0; ++i )
-    std::fprintf( stderr, "  %s", T_table[i].name );
-  std::fputs( "\nRotations are made counter-clockwise.\n", stderr );
+  if( verbosity >= 0 )
+    {
+    if( arg && std::strcmp( arg, "help" ) )
+      std::fprintf( stderr,"%s: bad bitmap trasformation `%s'\n", program_name, arg );
+    std::fputs( "Valid transformation names are:", stderr );
+    for( int i = 0; T_table[i].name != 0; ++i )
+      std::fprintf( stderr, "  %s", T_table[i].name );
+    std::fputs( "\nRotations are made counter-clockwise.\n", stderr );
+    }
   }
 
 
