@@ -1,5 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,11 +22,11 @@ public:
   enum Type { left, top, right, bottom, height, width };
 
 private:
-  const Bitmap * _bitmap;	// Bitmap to witch this profile belongs
+  const Bitmap * bitmap_;	// Bitmap to witch this profile belongs
   Type type;
-  int _limit, _max, _min, _mean;
-  signed char _isconcave, _isconvex, _isflat, _isflats,
-              _ispit, _istpit, _isupit, _isvpit, _istip;
+  int limit_, max_, min_, mean_;
+  signed char isconcave_, isconvex_, isflat_, isflats_,
+              ispit_, istpit_, isupit_, isvpit_, istip_;
   std::vector< int > data;
   void initialize() throw();
   int mean() throw();
@@ -33,9 +34,9 @@ private:
 public:
   Profile( const Bitmap & b, Type t ) throw();
 
-//  const Bitmap * bitmap() const throw() { return _bitmap; }
+//  const Bitmap * bitmap() const throw() { return bitmap_; }
 
-  int limit() throw() { if( _limit < 0 ) initialize(); return _limit; }
+  int limit() throw() { if( limit_ < 0 ) initialize(); return limit_; }
   int max() throw();
   int max( int l, int r = -1 ) throw();
   int min() throw();
@@ -43,7 +44,7 @@ public:
   int operator[]( int i ) throw();
   int pos( int p ) throw() { return ( ( samples() - 1 ) * p ) / 100; }
   int range() throw() { return max() - min(); }
-  int samples() throw() { if( _limit < 0 ) initialize(); return data.size(); }
+  int samples() throw() { if( limit_ < 0 ) initialize(); return data.size(); }
 
   int  area( int l = 0, int r = -1 ) throw();
   bool increasing( int i = 1 ) throw();
