@@ -1,9 +1,10 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+    Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cctype>
@@ -21,7 +21,7 @@
 #include "ucs.h"
 
 
-int UCS::base_letter( int code ) throw()
+int UCS::base_letter( const int code ) throw()
   {
   switch( code )
     {
@@ -89,7 +89,7 @@ int UCS::base_letter( int code ) throw()
   }
 
 
-int UCS::compose( int letter, int accent ) throw()
+int UCS::compose( const int letter, const int accent ) throw()
   {
   switch( letter )
     {
@@ -155,33 +155,49 @@ int UCS::compose( int letter, int accent ) throw()
   }
 
 
-bool UCS::isalnum( int code ) throw()
+bool UCS::isalnum( const int code ) throw()
   {
   return ( UCS::isalpha( code ) || UCS::isdigit( code ) );
   }
 
 
-bool UCS::isalpha( int code ) throw()
+bool UCS::isalpha( const int code ) throw()
   {
   return ( ( code < 128 && std::isalpha( code ) ) || base_letter( code ) );
   }
 
 
-bool UCS::isdigit( int code ) throw()
+bool UCS::isdigit( const int code ) throw()
   {
   return ( code <= '9' && code >= '0' );
   }
 
 
-bool UCS::islower( int code ) throw()
+bool UCS::islower( const int code ) throw()
   {
   if( code < 128 && std::islower( code ) ) return true;
-  code = base_letter( code );
-  return ( code && std::islower( code ) );
+  const int base = base_letter( code );
+  return ( base && std::islower( base ) );
   }
 
 
-bool UCS::islower_small( int code ) throw()
+bool UCS::islower_ambiguous( const int code ) throw()
+  {
+  if( islower_small_ambiguous( code ) ) return true;
+  switch( code )
+    {
+    case 'k': case 'p': case SCCEDI:
+    case SIGRAVE: case SIACUTE: case SICIRCU: case SIDIAER:
+    case SOGRAVE: case SOACUTE: case SOCIRCU: case SOTILDE: case SODIAER:
+    case SUGRAVE: case SUACUTE: case SUCIRCU: case SUDIAER:
+    case  SSCEDI: case SSCARON: case SZCARON:
+      return true;
+    default : return false;
+    }
+  }
+
+
+bool UCS::islower_small( const int code ) throw()
   {
   if( code >= 128 || !std::islower( code ) ) return false;
   switch( code )
@@ -194,7 +210,7 @@ bool UCS::islower_small( int code ) throw()
   }
 
 
-bool UCS::islower_small_ambiguous( int code ) throw()
+bool UCS::islower_small_ambiguous( const int code ) throw()
   {
   if( code >= 128 || !std::islower( code ) ) return false;
   switch( code )
@@ -206,17 +222,17 @@ bool UCS::islower_small_ambiguous( int code ) throw()
   }
 
 
-bool UCS::isspace( int code ) throw()
+bool UCS::isspace( const int code ) throw()
   {
   return ( code < 128 && std::isspace( code ) );
   }
 
 
-bool UCS::isupper( int code ) throw()
+bool UCS::isupper( const int code ) throw()
   {
   if( code < 128 && std::isupper( code ) ) return true;
-  code = base_letter( code );
-  return ( code && std::isupper( code ) );
+  const int base = base_letter( code );
+  return ( base && std::isupper( base ) );
   }
 
 
@@ -229,7 +245,7 @@ bool UCS::isvowel( int code ) throw()
   }
 
 
-char UCS::map_to_byte( int code ) throw()
+unsigned char UCS::map_to_byte( const int code ) throw()
   {
   if( code < 0 ) return 0;
   if( code < 256 ) return code;
@@ -251,7 +267,7 @@ char UCS::map_to_byte( int code ) throw()
   }
 
 
-char * UCS::ucs_to_utf8( int code ) throw()
+const char * UCS::ucs_to_utf8( const int code ) throw()
   {
   static char s[7];
 
@@ -274,7 +290,7 @@ char * UCS::ucs_to_utf8( int code ) throw()
   }
 
 
-int UCS::to_nearest_digit( int code ) throw()
+int UCS::to_nearest_digit( const int code ) throw()
   {
   switch( code )
     {
@@ -305,7 +321,7 @@ int UCS::to_nearest_digit( int code ) throw()
   }
 
 
-int UCS::to_nearest_letter( int code ) throw()
+int UCS::to_nearest_letter( const int code ) throw()
   {
   switch( code )
     {
@@ -323,7 +339,7 @@ int UCS::to_nearest_letter( int code ) throw()
   }
 
 
-int UCS::toupper( int code ) throw()
+int UCS::toupper( const int code ) throw()
   {
   if( code < 128 ) return std::toupper( code );
   switch( code )
